subroutine Base2Phase(Tempr,Presr,Choice,Answer)
! Purpose: To calculate the value of whatever state parameter is 
!          selected on the upper boundary of the two-phase region
!          for a given temperature (C) and pressure (Pa)
! Input: Temperature (C), Pressure (Pa), Menu Selection:
!        Choice should be a letter between A and D. It selects from 
!        the following menu of parameters to be calculated:
!        A - liquid density on the upper boundary of 2-phase region
!        B - vapor density  on the upper boundary of 2-phase region
!        C - liquid salinity on the upper boundary of 2-phase region
!        D - vapor salinity on the upper boundary of 2-phase region

! Output: Answer, the relevant property on the two-phase surface,
! depending on the letter to which Choice is set (see above).

! For 300 C <= T <= 800 C, the table values between which this 
! program interpolates were compiled from a program written by 
! Andrzej Anderko based upon the theoretical framework from his 
! and Kenneth Pitzer's 1993 Geochim. paper (see below).

! For T<300 C, the pressures within the range of validity for
! this code's equations (85 bars <= p <= 1000 bars) of state are 
! all above the critical curve, so that boundary values for the 
! two-phase region are not needed.

use CommonVars, only: Prec,TempArray ,PresMat2Ph
implicit none
real(selected_real_kind(Prec)) :: Tempr,Presr,Answer,Val11, &
   Val12,Val21,Val22,P11,P12,P21,P22,Val1,Val2,TempFrac, &
   T1,T2,Slope1,Slope2,PMap1,PMap2,PresFrac,Pres3Phase, &
   CritiPres
character :: Choice
integer :: Row1,Row2,Col11,Col12,Col21,Col22

call FindTemp(Tempr,Row1,Row2)
T1 = TempArray(Row1)
T2 = TempArray(Row2)
Row1 = Row1-30
Row2 = Row2-30
TempFrac=(Tempr-T1)/(T2-T1)
PresFrac = (Presr-Pres3Phase(Tempr))/ &
           (CritiPres(Tempr)-Pres3Phase(Tempr))
PMap1 = (1.D0-PresFrac)*Pres3Phase(T1)+PresFrac*CritiPres(T1)
PMap2 = (1.D0-PresFrac)*Pres3Phase(T2)+PresFrac*CritiPres(T2)
call FindPres2Ph(PMap1,Row1,Col11,Col12)
call FindPres2Ph(PMap2,Row2,Col21,Col22)
call Get2PhaseResult(Row1,Col11,Choice,Val11)
call Get2PhaseResult(Row1,Col12,Choice,Val12)
call Get2PhaseResult(Row2,Col21,Choice,Val21)
call Get2PhaseResult(Row2,Col22,Choice,Val22)
P11 = PresMat2Ph(Row1,Col11)*1.D5
P12 = PresMat2Ph(Row1,Col12)*1.D5
P21 = PresMat2Ph(Row2,Col21)*1.D5
P22 = PresMat2Ph(Row2,Col22)*1.D5
if (P12.ne.P11) then
   Slope1 = (Val12-Val11)/(P12-P11)
   Val1 = Slope1*(PMap1-P11)+Val11
else
   Val1 = Val11
end if
if (P22.ne.P21) then
   Slope2 = (Val22-Val21)/(P22-P21)
   Val2 = Slope2*(PMap2-P21)+Val21
else
   Val2 = Val21
end if

Answer = (1.D0-TempFrac)*Val1+TempFrac*Val2
end

! Anderko A, Pitzer KS (1993) equation-of-state representation of 
!      phase equilibria and volumetric properties of the system 
!      NaCl-H2O above 573 K,Geochim. Cosmochim. Acta,57,1657-1680
