program FISHES
! Fully Implicit Seafloor Hydrothermal Event Simulator
! Written by Kayla Lewis

! A note on the units: SI units have been used as much as
! possible, so pressure is always in Pa, time is in s unless
! otherwise stated, enthalpy is in joules/kg, density is in
! kg/m^3, and salinity is in weight fraction of NaCl - that 
! is, the wt% NaCl divided by 100. Note however, that the input
! files should have the salinity in wt% NaCl.

! CommonVars contains several shared variables, no variables
! in this code are named implicitly, and for every module,
! the only variables available to it from CommonVars are those
! that it specifically needs.
use CommonVars, only:Time,TotalTime,Delta_t,Delta_t_Array,           &
   NumOf_t_Periods,tPeriodLengths,Prec,TimeStepIncr,PrintFreqArray,  &
   SuppressScreen,LineByLineSwitch,BoundValGradual,                  &
   NumNodesX,NumNodesY,NumNodesZ,LogFile,TimeStepFlag,               &
   ComputeStaticPres,NumTimeCuts,MaxTimeCuts,TimeStepCheck,Unity,    &
   Zero,MaxDelta_t_Array,MinDelta_t_Array,SourceAdjust,OldDelta_t,   &
   Pres,SaltErrNum,StepNum,LastPrintTime,PrintFreq,tPeriodNum
implicit none
integer :: L,PhaseChange,I,J,K,ConvergedP,ConvergedT,ConvergedX,     &
   MinStepViolate
real (selected_real_kind(Prec)) :: NextPeriodTime,MaxDelta_t,        &
   MinDelta_t

! Read user defined parameters from the input files (InFile0 
! and InFile1)
call UserParams

write(LogFile,*) 'Loading thermodynamic lookup tables...'
call ReadEOSVals
write(LogFile,*) 'Tables loaded.'
write(LogFile,*)
! -------------------------------------------------------------------
!                       Initialization Section
! -------------------------------------------------------------------
! Place the fixed constant and upwind boundary values in place
call SetBoundaries
! Convert the salinities into fractions instead of percentages
call ConvertSali
! Time = time on the simulation clock.
Time = 0.D0
TotalTime = 0.D0
! NumTimeCuts = # of times in a row that the time step has been cut
NumTimeCuts = 0
! MinStepViolate = # of attemps to set the time step to a value 
! lower than the minimum allowed value MinDelta_t
MinStepViolate = 0
SourceAdjust = 0
TimeStepFlag = 0
SaltErrNum = 0
! StepNum = # time marching steps since start time
StepNum = 0
do I=1,NumOf_t_Periods
   TotalTime = TotalTime + tPeriodLengths(I)
end do
! TotalTime = the total time the simulation is to run.
if (ComputeStaticPres == 1) then
   call StaticPres
end if
if (BoundValGradual=='Y') then
   call BoundValGradIncr(Unity)
end if
call GetPhaseIndices(PhaseChange)
call ThermoProps
call RecordOldEnth
call RecordOldDens
call RecordOldPoros
call CalcInterface
call GetVelocities
call RecordOldPTX
call InterfMassFlux
call InterfSaltFlux
call InterfHeatFlux
call DensDerivs

tPeriodNum = 1
NextPeriodTime = tPeriodLengths(tPeriodNum)
PrintFreq = PrintFreqArray(tPeriodNum)
Delta_t = Delta_t_Array(tPeriodNum)
OldDelta_t = Delta_t
MaxDelta_t = MaxDelta_t_Array(tPeriodNum)
MinDelta_t = MinDelta_t_Array(tPeriodNum)
LastPrintTime = 0.D0
LineByLineSwitch=1

call MassTrackLocal(Unity)
call SaltTrackLocal(Unity)
call EnergyTrackLocal(Unity)
call MassTrackGlobal(Unity)
call SaltTrackGlobal(Unity)
call EnergyTrackGlobal(Unity)
! -------------------------------------------------------------------
write(LogFile,*) 'Temporal Marching Begun...'
! -------------------------------------------------------------------
!                       Temporal Marching Section
! -------------------------------------------------------------------
do while (Time<=TotalTime)

   call Update

   call GetTempCoeffs
   call SolveEquation('T')

   call EnergyTrackLocal(Zero)
   call EnergyTrackGlobal(Zero)

   call GetSaliCoeffs
   call SolveEquation('X')

   call SaltTrackLocal(Zero)
   call SaltTrackGlobal(Zero)

   call Update

   call DensDerivs
   call GetPresCoeffs
   call SolveEquation('P')

   call MassTrackLocal(Zero)
   call MassTrackGlobal(Zero)

   call RecordOldPTX
   call RecordOldEnth
   call RecordOldDens
   call RecordOldPoros

   if (BoundValGradual=='Y') then
      call BoundValGradIncr(Zero)
   end if
   !-----------------------------------------------------------------
   !                    Time Period Tracking
   !-----------------------------------------------------------------
   if (Time>NextPeriodTime) then
      tPeriodNum = tPeriodNum + 1
      Delta_t = Delta_t_Array(tPeriodNum)
      PrintFreq = PrintFreqArray(tPeriodNum)
      MaxDelta_t = MaxDelta_t_Array(tPeriodNum)
      MinDelta_t = MinDelta_t_Array(tPeriodNum)
      NextPeriodTime = NextPeriodTime + tPeriodLengths(tPeriodNum)
   end if
   if ((Time-LastPrintTime)>=PrintFreq) then
      ! Call the output subroutine to print the results
      call PrintOutput(LastPrintTime)
      LastPrintTime = Time
   end if
   !-----------------------------------------------------------------
   Time = Time + Delta_t
   StepNum = StepNum + 1
   !-----------------------------------------------------------------
   !                     Time Step Control
   !-----------------------------------------------------------------
   SourceAdjust = 0
   if (TimeStepFlag == 1) then
      NumTimeCuts = NumTimeCuts + 1
      if ((NumTimeCuts <= MaxTimeCuts).and. &
         (MinStepViolate<=MaxTimeCuts)) then
         write(LogFile,*) 'Attempting to halve time step'
         OldDelta_t = Delta_t
         SourceAdjust = 1
         Delta_t = MAX(MinDelta_t,Delta_t/2.D0)
         write(LogFile,*) 'New time step: ',Delta_t
         StepNum = 0
         if (Delta_t==OldDelta_t) then
            MinStepViolate = MinStepViolate + 1
         end if
      else
         if (NumTimeCuts > MaxTimeCuts) then
            write(LogFile,*) 'Too many time step cuts in a row'
         elseif (MinStepViolate>MaxTimeCuts) then
            write(LogFile,*) 'Too many attempts to set the time step'
            write(LogFile,*) 'below the minimum allowed value'
         end if
         write(LogFile,*) 'Halting'
         stop
      end if
   else
      NumTimeCuts = 0
   end if
   if (Delta_t*TimeStepIncr < MaxDelta_t) then
      if (TimeStepFlag==0) then
         OldDelta_t = Delta_t
         SourceAdjust = 1
      end if
      Delta_t = Delta_t*TimeStepIncr
   end if
   TimeStepFlag = 0
   !-----------------------------------------------------------------
end do
! -------------------------------------------------------------------
!                       End of Temporal Marching
! -------------------------------------------------------------------
write(LogFile,*) 'Temporal Marching Completed.'

if (SuppressScreen=='Y') then
   close(LogFile)
end if

end
