subroutine LineByLine(Coeffs,OldVals,CurrVals,Which,InertiaCoeff, &
   Converged)
! Purpose: Solves the system of equations determined from the
!          coefficients of the temperature, pressure, or salinity
!          equations.
! Input: The coefficient matrix, the array of old values of the 
!        variable of interest, and the choice of variable for 
!        which to solve: Which =
!        T - temperature
!        P - pressure
!        X - salinity
! Output: New values stored in CurrVals; Converged = 1 if
!         convergence was reached, 0 otherwise

use CommonVars, only: Prec,MaxX,MaxY,MaxZ,NumNodesX,                 &
   NumNodesY,NumNodesZ,SaliNodeTypes,PresNodeTypes,                  &
   TempNodeTypes,MaxLongArray,LineByLineSwitch,LogFile,              &
   MaxTempChange,MaxPresChange,MaxSaliChange,MaxPres,MinPres,        &
   MaxTemp,MinTemp,MaxSali,MinSali,MaxTimeCuts,NumTimeCuts,          &
   Delta_t,PresBoundVals,SaliBoundVals,TempBoundVals,                &
   TimeStepCheck,TimeStepFlag,MaxPIter,MaxTIter,PhaseIndices
implicit none
real (selected_real_kind(Prec)) :: Coeffs(MaxX,MaxY,MaxZ,8),         &
   RHSConst(MaxLongArray),aT(MaxLongArray),aB(MaxLongArray),         &
   aN(MaxLongArray),aS(MaxLongArray),aW(MaxLongArray),               &
   aE(MaxLongArray),Sources(MaxLongArray),aP(MaxLongArray),          &
   Solutions(MaxLongArray),CurrVals(MaxX,MaxY,MaxZ),                 &
   OldVals(MaxX,MaxY,MaxZ),PrevIterVals(MaxX,MaxY,MaxZ),             &
   InitDiffs(MaxX,MaxY,MaxZ),CurrDiffs(MaxX,MaxY,MaxZ),              &
   ConvergeTol,CutOff,Inertia,InertiaCoeff
     
integer :: Iter,I,J,K,Start,Finish,Increment,Flag,                   &
     WestEastCount,MaxIter,Converged
character :: Which

Converged = 1
if (Which=='P') then
   ConvergeTol = 1.D-1
   CutOff = 1.D-9
else
   ConvergeTol = 1.D-1
   CutOff = 1.D-3
end if
Iter = 1
Flag = 0
WestEastCount = 1
if (LineByLineSwitch==1) then
   ! The Purpose of LineByLineSwitch is to toggle between
   ! 1 and -1 once each time LineByLine is called; the first
   ! sweeps will have directions determined by whether this
   ! variable is -1 or 1. Done to preserve symmetry - see
   ! below comment for WestEastCount as well.
   LineByLineSwitch = -1
elseif (LineByLineSwitch==-1) then
   LineByLineSwitch = 1
end if
do while (Flag==0)
!--------------------------------------------------------------
!                       West-east passes
!-------------------------------------------------------------
if (WestEastCount<3) then
   ! The purpose of using WestEastCount is to ensure that 
   ! the order of sweeps is west, west, east, east, so that
   ! when the west-east passes are combined with the top-
   ! bottom passes, up sweeps will be alternately paired with
   ! west and east sweeps, and analogously for the downward
   ! sweeps. This approach should help to maintain symmetry
   ! numerically in situations that are physically symmetric.
   Increment = 1/LineByLineSwitch
   WestEastCount = WestEastCount + 1
elseif (WestEastCount>=3) then
   Increment = -1/LineByLineSwitch
   WestEastCount = WestEastCount + 1
   if (WestEastCount==5) then
      WestEastCount = 1
   end if
end if

if (Increment<0) then
   Start = NumNodesX
   Finish = 1
else
   Start = 1
   Finish = NumNodesX
end if

if (Iter>2) then
   Flag = 1
end if

do I=Start,Finish,Increment
   do J=1,NumNodesY
      do K=1,NumNodesZ
         if (Which=='X') then
            Inertia=0.D0
         else
            Inertia = InertiaCoeff*Coeffs(I,J,K,1)
         end if
         if (Iter==1) then
            PrevIterVals(I,J,K) = OldVals(I,J,K)
         end if
         aT(K)=Coeffs(I,J,K,7)
         aB(K)=Coeffs(I,J,K,6)
         aW(K)=Coeffs(I,J,K,4)
         aE(K)=Coeffs(I,J,K,5)
         aN(K)=Coeffs(I,J,K,2)
         aS(K)=Coeffs(I,J,K,3)
         aP(K)=Coeffs(I,J,K,1)+Inertia
         Sources(K)=Coeffs(I,J,K,8)+Inertia*PrevIterVals(I,J,K)
         if ((Iter==1).and.(Which.ne.'X')) then
            if ((aT(K)<0.D0).or.(aB(K)<0.D0).or.(aW(K)<0.D0).or. &
               (aE(K)<0.D0).or.(aN(K)<0.D0).or.(aS(K)<0.D0).or. &
               (aP(K)<0.D0)) then
               write(LogFile,*) 'One or more coefficients are negative'
               write(LogFile,*) 'for the ',Which,' equation'
            end if
         end if
         if (I==1) then
            RHSConst(K) = CurrVals(I+1,J,K)*aE(K)+Sources(K)
         elseif (I==NumNodesX) then
            RHSConst(K) = CurrVals(I-1,J,K)*aW(K)+Sources(K)
         else
            RHSConst(K) = CurrVals(I-1,J,K)*aW(K)+ &
                          CurrVals(I+1,J,K)*aE(K)+ Sources(K)
         end if
      end do
      ! Prepare the diagonals for TriDiMatSolv
      aT(1) = 0.D0
      aB(NumNodesZ) = 0.D0
      aP(NumNodesZ+1) = -100.D0
      RHSConst(NumNodesZ+1) = -100.D0
      aT(NumNodesZ+1) = -100.D0
      aB(NumNodesZ+1) = -100.D0
      call TriDiMatSolv(aT,aP,aB,RHSConst,Solutions)
      ! Store the new results
      do K=1,NumNodesZ
         if (Which=='X') then
            if (SaliNodeTypes(I,J,K).ne.'C') then
               CurrVals(I,J,K)= Solutions(K)
               if (CurrVals(I,J,K)>MaxSali) then
                  CurrVals(I,J,K)=MaxSali
               elseif (CurrVals(I,J,K)<MinSali) then
                  CurrVals(I,J,K)=MinSali
               elseif ((CurrVals(I,J,K)<=MaxSali).and. &
                       (CurrVals(I,J,K)>=MinSali)) then
                  if (DABS(CurrVals(I,J,K)- &
                     OldVals(I,J,K))>MaxSaliChange) then
                     CurrVals(I,J,K) = OldVals(I,J,K)
                  end if
               else ! Salinity is NaN
                  CurrVals(I,J,K)=OldVals(I,J,K)
               end if
            else
               CurrVals(I,J,K) = SaliBoundVals(I,J,K)
            end if
         elseif (Which=='T') then
            if (TempNodeTypes(I,J,K).ne.'C') then
               CurrVals(I,J,K)= Solutions(K)
               if (CurrVals(I,J,K)>MaxTemp) then
                  CurrVals(I,J,K)=MaxTemp
               elseif (CurrVals(I,J,K)<MinTemp) then
                  CurrVals(I,J,K)=MinTemp
               elseif ((CurrVals(I,J,K)<=MaxTemp).and. &
                       (CurrVals(I,J,K)>=MinTemp)) then
                  if (DABS(CurrVals(I,J,K)- &
                     OldVals(I,J,K))>MaxTempChange) then
                     CurrVals(I,J,K) = OldVals(I,J,K)
                  end if
               else ! Temperature is NaN
                  CurrVals(I,J,K)=OldVals(I,J,K)
               end if
            else
               CurrVals(I,J,K) = TempBoundVals(I,J,K)
            end if
         elseif (Which=='P') then
            if (PresNodeTypes(I,J,K).ne.'C') then
               CurrVals(I,J,K)= Solutions(K)
               if (CurrVals(I,J,K)>MaxPres) then
                  CurrVals(I,J,K)=MaxPres
               elseif (CurrVals(I,J,K)<MinPres) then
                  CurrVals(I,J,K)=MinPres
               elseif ((CurrVals(I,J,K)<=MaxPres).and. &
                       (CurrVals(I,J,K)>=MinPres)) then
                  if (DABS(CurrVals(I,J,K)- &
                     OldVals(I,J,K))>MaxPresChange) then
                     CurrVals(I,J,K) = OldVals(I,J,K)
                  end if
               else ! Pressure is NaN
                  CurrVals(I,J,K)=OldVals(I,J,K)
               end if
            else
               CurrVals(I,J,K) = PresBoundVals(I,J,K)
            end if
         end if
         if (NumNodesX==1) then
           if (Iter==2) then
               InitDiffs(I,J,K) = DABS(CurrVals(I,J,K)-PrevIterVals(I,J,K))
           elseif (Iter>2) then
               CurrDiffs(I,J,K) = DABS(CurrVals(I,J,K)-PrevIterVals(I,J,K))
               if ((InitDiffs(I,J,K)>CutOff).and. &
                   ((CurrDiffs(I,J,K)/InitDiffs(I,J,K))>ConvergeTol)) then
                  Flag = 0
               end if
            end if
            PrevIterVals(I,J,K)=CurrVals(I,J,K)
         end if
      end do
   end do
end do
!--------------------------------------------------------------
!                       Bottom-top passes
!--------------------------------------------------------------
if ((NumNodesX>1).and.(Which.ne.'X')) then 
!--------------------
if (MODULO(Iter,2)==0.D0) then
   Increment = 1/LineByLineSwitch
else
   Increment = -1/LineByLineSwitch
end if

if (Increment<0) then
   Start = NumNodesZ
   Finish = 1
else
   Start = 1
   Finish = NumNodesZ
end if
!--------------------
do J=1,NumNodesY
   do K=Start,Finish,Increment
      do I=1,NumNodesX
         if (Which=='X') then
            Inertia = 0.D0
         else
            Inertia = InertiaCoeff*Coeffs(I,J,K,1)
         end if
         aT(I)=Coeffs(I,J,K,7)
         aB(I)=Coeffs(I,J,K,6)
         aW(I)=Coeffs(I,J,K,4)
         aE(I)=Coeffs(I,J,K,5)
         aN(I)=Coeffs(I,J,K,2)
         aS(I)=Coeffs(I,J,K,3)
         aP(I)=Coeffs(I,J,K,1)+Inertia
         Sources(I)=Coeffs(I,J,K,8)+Inertia*PrevIterVals(I,J,K)
         if ((Iter==1).and.(Which.ne.'X')) then
            if ((aT(I)<0.D0).or.(aB(I)<0.D0).or.(aW(I)<0.D0).or. &
               (aE(I)<0.D0).or.(aN(I)<0.D0).or.(aS(I)<0.D0).or. &
               (aP(I)<0.D0)) then
               write(LogFile,*) 'One or more coefficients are negative'
               write(LogFile,*) 'for the ',Which,' equation'
            end if
         end if
         if (K==1) then
            RHSConst(I) = CurrVals(I,J,K+1)*aB(I)+Sources(I)
         elseif (K==NumNodesZ) then
            RHSConst(I) = CurrVals(I,J,K-1)*aT(I)+Sources(I)
         else 
            RHSConst(I) = CurrVals(I,J,K-1)*aT(I)+ &
                          CurrVals(I,J,K+1)*aB(I)+ Sources(I)
         end if
      end do
      ! Prepare Diagonals for TriDiMatSolv
      aW(1) = 0.D0
      aE(NumNodesX) = 0.D0
      aW(NumNodesX+1) = -100.D0
      aE(NumNodesX+1) = -100.D0
      RHSConst(NumNodesX+1) = -100.D0
      aP(NumNodesX+1) = -100.D0
      call TriDiMatSolv(aW,aP,aE,RHSConst,Solutions)
      ! Store the new results
      do I=1,NumNodesX
         if (Which=='T') then
            if (TempNodeTypes(I,J,K).ne.'C') then
               CurrVals(I,J,K)= Solutions(I)
               if (CurrVals(I,J,K)>MaxTemp) then
                  CurrVals(I,J,K)=MaxTemp
               elseif (CurrVals(I,J,K)<MinTemp) then
                  CurrVals(I,J,K)=MinTemp
               elseif ((CurrVals(I,J,K)<=MaxTemp).and. &
                       (CurrVals(I,J,K)>=MinTemp)) then
                  if (DABS(CurrVals(I,J,K)- &
                     OldVals(I,J,K))>MaxTempChange) then
                     CurrVals(I,J,K) = OldVals(I,J,K)
                  end if
               else ! Temperature is NaN
                  CurrVals(I,J,K)=OldVals(I,J,K)
               end if
            else
               CurrVals(I,J,K) = TempBoundVals(I,J,K)
            end if
         elseif (Which=='P') then
            if (PresNodeTypes(I,J,K).ne.'C') then
               CurrVals(I,J,K)= Solutions(I)
               if (CurrVals(I,J,K)>MaxPres) then
                  CurrVals(I,J,K)=MaxPres
               elseif (CurrVals(I,J,K)<MinPres) then
                  CurrVals(I,J,K)=MinPres
               elseif ((CurrVals(I,J,K)<=MaxPres).and. &
                       (CurrVals(I,J,K)>=MinPres)) then
                  if (DABS(CurrVals(I,J,K)- &
                     OldVals(I,J,K))>MaxPresChange) then
                     CurrVals(I,J,K) = OldVals(I,J,K)
                  end if
               else ! Pressure is NaN
                  CurrVals(I,J,K)=OldVals(I,J,K)
               end if
            else
               CurrVals(I,J,K) = PresBoundVals(I,J,K)
            end if
         end if
         if (Iter==2) then
            InitDiffs(I,J,K) = DABS(CurrVals(I,J,K)-PrevIterVals(I,J,K))
         elseif (Iter>2) then
            CurrDiffs(I,J,K) = DABS(CurrVals(I,J,K)-PrevIterVals(I,J,K))
            if ((InitDiffs(I,J,K)>CutOff).and. &
                ((CurrDiffs(I,J,K)/InitDiffs(I,J,K))>ConvergeTol)) then
               Flag = 0
            end if
         end if
         PrevIterVals(I,J,K)=CurrVals(I,J,K)
      end do
   end do
end do
!----------------
end if
if (Which=='P') then
   MaxIter = MaxPIter
elseif (Which=='T') then
   MaxIter = MaxTIter
elseif (Which=='X') then
   MaxIter = 1
   Flag = 1
end if
if (Iter>MaxIter) then
   Flag = 1
   Converged = 0
end if
Iter = Iter + 1
!--------------------------------------------------------------------
! Note: the total number of iterations at this point is Iter-1
!--------------------------------------------------------------------
end do
!write(LogFile,*) Which,Iter-1
end
