subroutine SolveEquation(Which)
! Purpose: to solve the equation identified by Which, 'P' for the
!          pressure equation, 'T' for the temperature equation, and
!          'S' for the salinity equation.

use CommonVars, only: Prec,PresCoeffs,TempCoeffs,SaliCoeffs,       &
   NumNodesX,NumNodesY,NumNodesZ,Pres,Temp,Sali,OldSali,           &
   OldPres,OldTemp,MaxX,MaxY,MaxZ,LineByLineSwitch,LogFile
implicit none
real(selected_real_kind(Prec)) :: PassTwoVals(MaxX,MaxY,MaxZ),     &
   PassOneVals(MaxX,MaxY,MaxZ),InertiaCoeff,StartInertia
integer :: I,J,K,Converged,Counter,MaxCount,Flag
character :: Which

! The salinity equation does not require LineByLine to make multiple
! passes or "sweeps", but the temperature and pressure equations do.
! LineByLine has a variable, LineByLineSwitch, that toggles between
! -1 and 1 each time LineByLine is called, reversing the directions
! of the initial top-bottom and west-east sweeps. By calling 
! LineByLine twice and averaging the results, biasing of the 
! solutions by the initial sweep directions is prevented; that is 
! the purpose of the variables PassOneVals and PassTwoVals.

MaxCount = 3
if (Which=='X') then
   InertiaCoeff = 0.D0
   ! NOTE: When LineByLine is called with equation type 'X', the input
   ! inertia intentionally has no effect - relaxation applied to 'X' 
   ! can cause the evolution of salinity to be time inaccurate
   call LineByLine(SaliCoeffs,OldSali,Sali,Which, &
                   InertiaCoeff,Converged)
elseif ((Which=='T').or.(Which=='P')) then
   if (Which=='P') then
      InertiaCoeff = 1.D-2
   elseif (Which=='T') then
      InertiaCoeff = 1.D0
   end if
   StartInertia = InertiaCoeff
   Converged = 0
   Counter = 0
   Flag = 0
   do while ((Converged==0).and.(Flag==0))
      if (Which=='P') then
         call LineByLine(PresCoeffs,OldPres,PassOneVals,Which, &
                         InertiaCoeff,Converged)
      elseif (Which=='T') then
         call LineByLine(TempCoeffs,OldTemp,PassOneVals,Which, &
                         InertiaCoeff,Converged)
      end if
      if (Converged==0) then
         Counter = Counter + 1
         if (Counter==1) then
            InertiaCoeff = 1.D-1*StartInertia
         elseif (Counter==2) then
            InertiaCoeff = 10.D0*StartInertia
         else
            InertiaCoeff = 10.D0*InertiaCoeff
         end if
         !write(LogFile,*) 'no convergence for ',Which,' on try ', &
         !   Counter
         !write(LogFile,*) 'increasing underrelaxation...'
         if (Counter > MaxCount) then
            Flag = 1
            write(LogFile,*) 'no convergence for ',Which
            write(LogFile,*) 'halting'
            STOP
         end if
      end if
      !if ((Counter>1).and.(Converged==1)) then
      !   write(LogFile,*) 'convergence for ',Which,' succeeded ', &
      !      'on try ',Counter
      !end if
   end do
   Converged = 0
   Counter = 0
   Flag = 0
   do while ((Converged==0).and.(Flag==0))
      if (Which=='P') then
         call LineByLine(PresCoeffs,OldPres,PassTwoVals,Which, &
                         InertiaCoeff,Converged)
      else if (Which=='T') then
         call LineByLine(TempCoeffs,OldTemp,PassTwoVals,Which, &
                         InertiaCoeff,Converged)
      end if
      if (Converged==0) then
         Counter = Counter + 1
         if (Counter==1) then
            InertiaCoeff = 1.D-1*StartInertia
         elseif (Counter==2) then
            InertiaCoeff = 10.D0*StartInertia
         else
            InertiaCoeff = 10.D0*InertiaCoeff
         end if
         !write(LogFile,*) 'no convergence for ',Which,' on try ', &
         !   Counter
         !write(LogFile,*) 'increasing underrelaxation...'
         if (Counter > MaxCount) then
            Flag = 1
            write(LogFile,*) 'no convergence for ',Which
            write(LogFile,*) 'halting'
            STOP
         end if
      end if
      !if ((Counter>1).and.(Converged==1)) then
      !   write(LogFile,*) 'convergence for ',Which,' succeeded ', &
      !      'on try ',Counter
      !end if
   end do
   do I=1,NumNodesX
      do J=1,NumNodesY
         do K=1,NumNodesZ
            if (Which=='P') then
               Pres(I,J,K)= &
                  (PassOneVals(I,J,K)+PassTwoVals(I,J,K))/2.D0
            elseif (Which=='T') then
               Temp(I,J,K)= &
                  (PassOneVals(I,J,K)+PassTwoVals(I,J,K))/2.D0
            end if
         end do
      end do
   end do
else
   write(LogFile,*) 'invalid value for variable Which - SolveEquation'
   write(LogFile,*) 'halting'
   STOP
end if

end
