subroutine UserParams
! Purpose: To read in all the user defined parameters from the input
!          files InFile0 and InFile1

use CommonVars, only: NumNodesX,NumNodesY,NumNodesZ,               &
    Time,TimeStepIncr,RockDens,CpRock,RockThermCond,               &
    Grav,HaliteDens,CpHalite,FluidThermCond,SaltChemDiff,          &
    PrintFreqArray,tPeriodLengths,Delta_t_Array,NumOf_t_Periods,   &
    SuppressScreen,Dispersivity,CorrelationLength,BoundValGradual, &
    BoundSide,EquationType,StartBoundNode,EndBoundNode,            &
    IncreaseAmount,Tilt,Pi,IncreaseTime,LogFile,TimeStepCheck,     &
    MaxDelta_t_Array,MinDelta_t_Array,MaxX,MaxY,MaxZ,MaxLongArray, &
    MaxPIter,MaxTIter

implicit none
integer :: I
character :: FileName*80

! open main input and output files
FileName='InFile0'
call OpenFile(FileName)
! unit 7 is InFile0

FileName='OutFile0'
call OpenFile(FileName)
! unit 8 is OutFile0

read (7,*)
read (7,*) NumNodesX,NumNodesY,NumNodesZ,SuppressScreen
! NumNodesX - the number of nodes in the X direction, and so on.
! SuppressScreen - whether to suppress screen output while running
! Grav - gravity, always in the Z direction (m/s^2)

if (SuppressScreen.ne.'Y') then
   LogFile = 6
else
   LogFile = 80
   open(unit=LogFile,file='FISHES.log',status='replace')
end if

write(8,'(/,A)') &
      '--- Dimensions of the Problem Domain ---'
write(8,405) NumNodesX,NumNodesY,NumNodesZ
405 format (10X,'Number of:  nodes in the X direction = ',I5,/,&
            22X,'nodes in the Y direction = ',I5,/, &
            22X,'nodes in the Z direction = ',I5)

if ((NumNodesX > MaxX).or.(NumNodesY > MaxY).or. &
   (NumNodesZ > MaxZ)) then
   write(LogFile,*) 'Number of nodes in some direction exceeds the'
   write(LogFile,*) 'maximum allocated memory space. Consider'
   write(LogFile,*) 'setting MaxX, MaxY, or MaxZ to a larger value.'
   write(LogFile,*) 'These variables are located in CommonVars.f90'
   write(LogFile,*) 'Terminating'
   stop
end if

if ((MaxLongArray < MaxX).and.(MaxLongArray < MaxY).and. &
    (MaxLongArray < MaxZ)) then
   write(LogFile,*) 'WARNING:'
   write(LogFile,*) 'MaxLongArray in CommonVars.f90 should be set'
   write(LogFile,*) 'to the maximum of MaxX, MaxY, and MaxZ, which'
   write(LogFile,*) 'are located in the same file'
end if

write(8,*)
write(8,*) '--- Time, Dispersion, and Boundary Information ---'
write(8,'(10X,A)') 'All time variables in: seconds'

! TimeStepIncr - time step increase factor: the time step gets 
!                multiplied by this number after every time step 
read(7,'(1X)')
read(7,*) TimeStepIncr, Dispersivity, CorrelationLength

write(8,*) '         Factor by which to increase Delta_t after'  
write(8,*) '         each time step = ',TimeStepIncr
write(8,*) '         Dispersivity(unitless) = ',Dispersivity
write(8,*) '         Correlation Length(m) = ',CorrelationLength

read(7,*)
read(7,*) BoundValGradual,BoundSide
read(7,*) 
read(7,*) EquationType,StartBoundNode,EndBoundNode,IncreaseAmount
read(7,*) 
read(7,*) IncreaseTime,MaxPIter,MaxTIter

write(8,*) 
write(8,*) '         Gradually increase boundary value(s)? ', &
   BoundValGradual
write(8,*) '         Side of boundary = ',BoundSide
write(8,*) '         Equation type for boundary value(s) = ', &
   EquationType
write(8,*) '         Starting boundary node number = ',StartBoundNode
write(8,*) '         Ending boundary node number = ',EndBoundNode
write(8,*) '         Amount by which to increase = ',IncreaseAmount
write(8,*) '         Time over which to increase (s) = ',IncreaseTime
write(8,*) '         Maximum # of P iterations allowed = ',MaxPIter
write(8,*) '         Maximum # of T iterations allowed = ',MaxTIter
write(8,*)

read(7,*)
read(7,*) NumOf_t_Periods,TimeStepCheck
! NumOf_t_Periods - number of time periods to read from the bottom of OutFile0
read(7,*)
do I=1,NumOf_t_Periods
   read(7,*) tPeriodLengths(I),PrintFreqArray(I),Delta_t_Array(I), &
             MaxDelta_t_Array(I),MinDelta_t_Array(I)
end do

write(8,*) '--- Time Period and Time Step Info. ---'
write(8,*) '         Time step checking = ',TimeStepCheck
write(8,*) '         Number of time periods = ',NumOf_t_Periods
do I=1,NumOf_t_Periods
   write(8,*) '         Time period #',I
   write(8,*) '         Time period length (s)= ',tPeriodLengths(I)
   write(8,*) '         Print frequency (s) = ',PrintFreqArray(I)
   write(8,*) '         Time step size (s) = ',Delta_t_Array(I)
   write(8,*) '         Maximum time step allowed (s) = ',MaxDelta_t_Array(I)
   write(8,*) '         Minimum time step allowed (s) = ',MinDelta_t_Array(I)
end do

write(8,*)
write(8,*) '--- Rock Properties and Gravity --- '
read(7,*)
read(7,*) RockDens,CpRock,RockThermCond,Grav,Tilt

! RockDens - rock density in kg/m^3
! CpRock - rock specific heat at constant pressure
! RockThermCond - rock thermal conductivity (W/kg/C)
! Tilt - amount, in degrees clockwise from the horizontal, to tilt
!        the system.

write(8,*) '         Rock density (kg/m^3) = ',RockDens
write(8,*) '         Rock specific heat (@const. P) (J/kg/C) = ',CpRock
write(8,*) '         Rock thermal cond. (W/m/C) = ',RockThermCond
write(8,*) '         Gravity (m/s^2) = ',Grav,'  Tilt = ',Tilt
write(8,*)

! Convert Tilt to radians
Tilt = Tilt*Pi/180.D0

write(8,*) '--- Halite Properties and Misc ---'

! HaliteDens - halite density in kg/m^3
! CpHalite - specific heat of halite at constant pressure (W/kg/C)

read(7,*)
read(7,*) HaliteDens, CpHalite, FluidThermCond, SaltChemDiff

write(8,*) '         Halite density (kg/m^3) = ',HaliteDens
write(8,*) '         Halite specific heat (@const. P) (J/kg/C) = ',CpHalite
write(8,*) '         Fluid Thermal Conductivity (W/m/C) = ',FluidThermCond
write(8,*) '         Salt Chemical Diffusivity (kg/m/s) = ',SaltChemDiff

read(7,*)
call ReadInFile0Arrays

FileName = 'InFile1'
call OpenFile(FileName)
! InFile1 is now unit 9

call ReadInFile1Arrays

close(7)
close(8)
close(9)
end
